#!/usr/bin/env python

##Copyright (C) 2010 Louis Taylor  
## _    ___ _____                                                              
##| |  | _ \_   _|                                                             
##| |__|  _/ | |                                                               
##|____|_|   |_| *programus optimus est*                                       
##                                                                             
##This program is free software: you can redistribute it and/or modify         
##it under the terms of the GNU General Public License as published by         
##the Free Software Foundation, either version 3 of the License, or            
##(at your option) any later version.                                          
##                                                                             
##This program is distributed in the hope that it will be useful,              
##but WITHOUT ANY WARRANTY; without even the implied warranty of               
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                
##GNU General Public License for more details.                                 
##                                                                            
##You should have received a copy of the GNU General Public License            
##along with this program.  If not, see http://www.gnu.org/licenses/gpl-3.0.txt  

import gtk, gobject
import sys, os
import pynotify
import gconf
import taylortypeBackend

file = sys.argv[0]
path = os.path.split(file)[0]

version = '0.3alpha'

datapath = '/usr/share/taylortype/data/'
#initialize the paths to some icons
healthy_icon = os.path.join(datapath, 'gdu-smart-healthy.svg')
threshold_icon = os.path.join(datapath, 'gdu-smart-threshold.svg')
failing_icon = os.path.join(datapath, 'gdu-smart-failing.svg')
unknown_icon = os.path.join(datapath, 'gdu-smart-unknown.svg')
logo_icon = os.path.join('/usr/share/pixmaps', 'taylortype.svg')

class taylortype:
    def __init__(self):
        #start some variables:
        self.texfile = '' #get a default filename in case self.get_tex_file() fails
        self.output_dir = ''
        
        self.window = gtk.Window()
        self.window.hide()
        self.statusIcon = gtk.StatusIcon()
        self.statusIcon.set_from_file(healthy_icon)
        self.statusIcon.set_visible(True)
        self.statusIcon.set_tooltip_text('Recompile %s' % self.texfile)
        
        self.statusIcon.connect('popup-menu', self.OnPopup)
        self.statusIcon.connect('activate', self.OnOpen)
        
        self.menu = gtk.Menu()
        
        imenu = gtk.Menu()

        importm = gtk.ImageMenuItem(gtk.STOCK_PREFERENCES, 'preferences')
        importm.set_submenu(imenu)
        importm.show()
        self.menu.append(importm)
        
        MenuItem = gtk.MenuItem('Choose _LaTeX file')
        imenu.append(MenuItem)
        MenuItem.set_tooltip_text('''Choose the LaTeX (.tex) file to use when compiling''')
        MenuItem.show()
        MenuItem.connect('activate', self.OnPrefs)
        
        MenuItem = gtk.MenuItem('Choose output _directory')
        imenu.append(MenuItem)
        MenuItem.set_tooltip_text('''Choose the output directory for all files used while compiling (output, .aux and log files)''')
        MenuItem.show()
        MenuItem.connect('activate', self.OnOutputPrefs)

        self.bibtex_checkBox = gtk.CheckMenuItem('Use _BibTeX when compiling')
        imenu.append(self.bibtex_checkBox)
        self.bibtex_checkBox.set_active(True)
        self.bibtex_checkBox.show()
        self.bibtex_checkBox.connect("activate", self.on_bibtex_Checkbox)
        self.bibtex_checkBox.set_tooltip_text('''Execute the BibTex command before compiling with LaTeX.
If the document does not use any /cite commands, an error message will appear.''')

        self.checkBox = gtk.CheckMenuItem('_Open file after compiling')
        imenu.append(self.checkBox)
        self.checkBox.set_active(True)
        self.checkBox.show()
        self.checkBox.connect("activate", self.onCheckbox)
        self.checkBox.set_tooltip_text('Open output pdf file with evince after compiling')
        
        aboutMenu = gtk.ImageMenuItem(gtk.STOCK_ABOUT, 'About')
        self.menu.append(aboutMenu)
        aboutMenu.show()
        aboutMenu.connect('activate', self.OnAbout)

        self.sep = gtk.SeparatorMenuItem()
        self.menu.append(self.sep)
        self.sep.show()
        
        quitMenu = gtk.ImageMenuItem(gtk.STOCK_QUIT, 'Quit')
        self.menu.append(quitMenu)
        quitMenu.show()
        quitMenu.connect('activate', self.OnQuit)
        
        self.use_bibtex = False #do not use bibtex by default
        self.start_gconf()
        
        if self.texfile == None or self.output_dir == None:
            self.OnNotConfigured()

    def startNotes(self):
        pynotify.init('taylortype')
            
    def onError(self, error_list):
        '''what to do in an error.
        Pass it the output from taylortypebackend e.g. 5 part list.'''
        
        text = [] #variable to store lines of message
        
        #this all looks worse than it is.
        #it works out what to show in the message depending on what in is given.
        
        #make things a bit clearer:
        error_msg = error_list[1].strip()
        error_ln = error_list[2].strip()
        error_quote = error_list[3].strip() #strip the whitespace?
        
        if error_msg == '':
            error_type = 'An error has occurred'
        else:
            error_type = error_msg
        
        if error_ln == '':
            if error_quote == '':
                pass
            else:
                text += ['"%s"' % error_quote]
        else:
            if error_quote == '':
                text += ['Line %s' % error_ln]
            else:
                text += ['Line %s:\n"%s"' % (error_ln, error_quote)]
            
        text = '\n'.join(text)
        n = pynotify.Notification(error_type, text, gtk.STOCK_DIALOG_ERROR)
        n.set_timeout(1)
        n.show()
        
        return text
        
    def OnNotConfigured(self):
        '''what to do if there are no gconf keys for use, ie return None'''
        self.statusIcon.set_from_file(unknown_icon)
        self.statusIcon.set_tooltip_text('Please configure the LaTeX file and the output directory to use')
        n = pynotify.Notification('TaylorType not configured', 'Please configure the LaTeX input file\nand the output directory.', gtk.STOCK_DIALOG_WARNING)
        n.set_timeout(1)
        n.show()
    
    def OnPopup(self, widget, button, timeout):
        '''gtk callback when system tray icon is right-clicked'''
        self.menu.popup(None, None, gtk.status_icon_position_menu, button, timeout, data=self.statusIcon)
    
    def OnOpen(self, widget):
        '''gtk callback when system tray icon is left-clicked'''
        
        if self.texfile == None or self.output_dir == None:
            self.OnNotConfigured() #attempt to stop idiots...
            return None
            
        #do some test thingys to show it's working
        self.statusIcon.set_from_file(threshold_icon)
        self.statusIcon.set_tooltip_text('compiling...')
        
        ##the line which does the compiling                                                         #is this clear?
        error = taylortypeBackend.compile(self.texfile, self.output_dir, rmlog=True, movepdf=False, usebibtex=self.use_bibtex)
        #                                                                       #get the value stored in gconf whether or not to use bibtex
        if error[0] == 'success':
            self.statusIcon.set_tooltip_text('Recompile %s' % self.texfile)
            if self.checkBox.get_active():
                print 'opening %s with evince' % error[1]
                os.system('evince %s &' % error[1]) #error[1] is the path to the pdf file
            self.statusIcon.set_from_file(healthy_icon)
            self.statusIcon.set_blinking(False)
            print 'compiled!'
        else:
            self.statusIcon.set_from_file(failing_icon)
            message = self.onError(error)
            self.statusIcon.set_tooltip_text('''an error has occurred, please correct
%s''' % message)
            #self.statusIcon.set_blinking(True) #make it blink on error?
            
    def onCheckbox(self, widget):
        '''remember what state the checkbox is in'''
        if widget.active:
            #make it true
            self.set_bool_key('/apps/taylortype/system-tray-preferences/open-when-finished', 1)
        else:
            #make it false
            self.set_bool_key('/apps/taylortype/system-tray-preferences/open-when-finished', 0)
            
    def OnOutputPrefs(self, widget):
        '''gtk callback when output preferences item is selected'''
        chooser = gtk.FileChooserDialog(title='Choose direcory to save output to',
        action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            filename = chooser.get_filename()
            self.set_str_key('/apps/taylortype/system-tray-preferences/output', filename)
            self.output_dir = filename
        elif response == gtk.RESPONSE_CANCEL:
            pass
        chooser.destroy()
        
    def OnPrefs(self, widget):
        '''gtk callback when preferences item is selected'''
        chooser = gtk.FileChooserDialog(title='Choose LaTeX file to work on',
        action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
        
        filter = gtk.FileFilter()
        filter.set_name('tex files (*.tex)')
        filter.add_pattern('*.tex')
        chooser.add_filter(filter)
        
        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        chooser.add_filter(filter)

        #chooser.run()
        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            filename = chooser.get_filename()
            self.set_str_key('/apps/taylortype/system-tray-preferences/filename', filename)
            self.texfile = filename
        elif response == gtk.RESPONSE_CANCEL:
            pass

        chooser.destroy()
        
    def OnAbout(self, widget):
        '''gtk callback when about item is selected'''
        about = gtk.AboutDialog()
        about.set_program_name('TaylorType')
        about.set_version(version) #use the version variable initilized above
        about.set_copyright('(c) Louis Taylor')
        about.set_comments('TaylorType is a program which speeds up the creation of LaTeX documents.')
        about.set_website('http://taylortype.sourceforge.net/')
        about.set_website_label('website')
        about.set_logo(gtk.gdk.pixbuf_new_from_file(logo_icon))
        about.set_authors(['Louis Taylor (main programing)<louis@taylor.bot.nu>', 'A.V. Tejesh (website) <vtejesh@users.sourceforge.net>'])
        about.set_artists(['Louis Taylor (icons) <louis@taylor.bot.nu>'])
        about.set_wrap_license(True)
        
        #should this be hard-coded?
        about.set_license('''
Copyright (C) 2010 Louis Taylor
This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in  the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>.
''')
        about.run()
        about.destroy()
        
    #TODO
    #make all these crazy client = gconf.client_get_default()'s become joined together
    #NOTE: sort of done now
    def start_gconf(self):
        '''Handles gconf changes, if any gconf variable is changed, a
        different method is called to handle this change.'''
        client = gconf.client_get_default()
        
        client.add_dir('/apps/taylortype/system-tray-preferences/', gconf.CLIENT_PRELOAD_NONE)
        client.notify_add('/apps/taylortype/system-tray-preferences/open-when-finished', self.onCheckbox_gconf)
        self.onCheckbox_gconf(client)
        
        client.add_dir('/apps/taylortype/system-tray-preferences/', gconf.CLIENT_PRELOAD_NONE)
        client.notify_add('/apps/taylortype/system-tray-preferences/filename', self.texfile_gconf)
        self.texfile_gconf(client)
        
        client.add_dir('/apps/taylortype/system-tray-preferences/', gconf.CLIENT_PRELOAD_NONE)
        client.notify_add('/apps/taylortype/system-tray-preferences/output', self.texfile_gconf)
        self.output_dir_gconf(client)
        
        client.add_dir('/apps/taylortype/system-tray-preferences/', gconf.CLIENT_PRELOAD_NONE)
        client.notify_add('/apps/taylortype/system-tray-preferences/use-bibtex', self.on_bibtex_gconf)
        self.on_bibtex_gconf(client)

    def set_bool_key(self, key, state):
        '''sets the gconf state of the key given'''
        client = gconf.client_get_default()
        client.set_bool(key, state)
            
    def onCheckbox_gconf(self, client, *args, **kwargs):
        '''set the checkbox if the gconf key is changed'''
        state = client.get_bool('/apps/taylortype/system-tray-preferences/open-when-finished')
        self.checkBox.set_active(state)

    def set_str_key(self, key, value):
        '''sets the gconf state of the key given'''
        client = gconf.client_get_default()
        client.set_string(key, value)
        
    def texfile_gconf(self, client, *args, **kwargs):
        '''get the filename to use when the gconf key is changed'''
        state = client.get_string('/apps/taylortype/system-tray-preferences/filename')
        self.texfile = state
        print self.texfile
        
    def output_dir_gconf(self, client, *args, **kwargs):
        '''get the output_dir to use when the gconf key is changed'''
        state = client.get_string('/apps/taylortype/system-tray-preferences/output')
        self.output_dir = state
            
    def on_bibtex_gconf(self, client, *args, **kwargs):
        '''set the checkbox if the gconf key is changed for bibtex checkbox'''
        state = client.get_bool('/apps/taylortype/system-tray-preferences/use-bibtex')
        self.bibtex_checkBox.set_active(state)
        
        self.use_bibtex = state
        
    def save_bibtex_gconf(self, widget):
        '''remember what state the bibtex checkbox is in'''
        if widget.active:
            #make it true
            self.set_bool_key('/apps/taylortype/system-tray-preferences/use-bibtex', 1)
        else:
            #make it false
            self.set_bool_key('/apps/taylortype/system-tray-preferences/use-bibtex', 0)
            
    def on_bibtex_Checkbox(self, widget):
        self.save_bibtex_gconf(widget)
        self.use_bibtex = widget.active
        
    def OnQuit(self, widget):
        '''gtk callback when quit menu item is selected'''
        print 'closing now, bye!'
        self.window.destroy()
        gtk.main_quit()
    
        
taylortype = taylortype()
gtk.main()
